@@ -5,7 +5,7 @@ from math import inf
 
 from ..css import computed_from_cascaded
 from ..css.computed_values import character_ratio, strut_layout
-from ..formatting_structure import boxes
+from ..formatting_structure import boxes, build
 from ..text.line_break import can_break_text, create_layout, split_first_line
 from .absolute import AbsolutePlaceholder, absolute_layout
 from .flex import flex_layout
@@ -169,8 +169,8 @@ def get_next_linebox(context, linebox, position_y, bottom_space, skip_stack,
             fixed_boxes, bottom_space, skip_stack=None)
         float_children.append(new_waiting_float)
         if waiting_float_resume_at:
-            context.broken_out_of_flow.append(
-                (waiting_float, containing_block, waiting_float_resume_at))
+            context.broken_out_of_flow[new_waiting_float] = (
+                waiting_float, containing_block, waiting_float_resume_at)
     if float_children:
         line.children += tuple(float_children)
 
@@ -180,7 +180,7 @@ def get_next_linebox(context, linebox, position_y, bottom_space, skip_stack,
 def skip_first_whitespace(box, skip_stack):
     """Return ``skip_stack`` to start just after removable leading spaces.
 
-    See http://www.w3.org/TR/CSS21/text.html#white-space-model
+    See https://www.w3.org/TR/CSS21/text.html#white-space-model
 
     """
     if skip_stack is None:
@@ -338,6 +338,7 @@ def first_letter_to_box(box, skip_stack, first_letter_style):
                             box.element, first_letter)
                         line_box.children = (text_box,)
                         box.children = (letter_box,) + tuple(box.children)
+                    build.process_text_transform(text_box)
                     if skip_stack and child_skip_stack:
                         index, = skip_stack
                         (child_index, grandchild_skip_stack), = (
@@ -382,12 +383,12 @@ def inline_block_box_layout(context, box, position_x, skip_stack,
 
     resolve_percentages(box, containing_block)
 
-    # http://www.w3.org/TR/CSS21/visudet.html#inlineblock-width
+    # https://www.w3.org/TR/CSS21/visudet.html#inlineblock-width
     if box.margin_left == 'auto':
         box.margin_left = 0
     if box.margin_right == 'auto':
         box.margin_right = 0
-    # http://www.w3.org/TR/CSS21/visudet.html#block-root-margin
+    # https://www.w3.org/TR/CSS21/visudet.html#block-root-margin
     if box.margin_top == 'auto':
         box.margin_top = 0
     if box.margin_bottom == 'auto':
@@ -397,10 +398,11 @@ def inline_block_box_layout(context, box, position_x, skip_stack,
 
     box.position_x = position_x
     box.position_y = 0
-    box, _, _, _, _ = block_container_layout(
+    box, _, _, _, _, _ = block_container_layout(
         context, box, bottom_space=-inf, skip_stack=skip_stack,
         page_is_empty=True, absolute_boxes=absolute_boxes,
-        fixed_boxes=fixed_boxes, adjoining_margins=None, discard=False)
+        fixed_boxes=fixed_boxes, adjoining_margins=None, discard=False,
+        max_lines=None)
     box.baseline = inline_block_baseline(box)
     return box
 
@@ -410,7 +412,7 @@ def inline_block_baseline(box):
 
     Position is taken from the top of its margin box.
 
-    http://www.w3.org/TR/CSS21/visudet.html#propdef-vertical-align
+    https://www.w3.org/TR/CSS21/visudet.html#propdef-vertical-align
 
     """
     if box.is_table_wrapper:
@@ -505,7 +507,7 @@ def split_inline_level(context, box, position_x, max_x, bottom_space,
     elif isinstance(box, boxes.InlineFlexBox):
         box.position_x = position_x
         box.position_y = 0
-        for side in ['top', 'right', 'bottom', 'left']:
+        for side in ('top', 'right', 'bottom', 'left'):
             if getattr(box, f'margin_{side}') == 'auto':
                 setattr(box, f'margin_{side}', 0)
         new_box, resume_at, _, _, _ = flex_layout(
@@ -530,7 +532,7 @@ def _out_of_flow_layout(context, box, containing_block, index, child,
         child.position_x = position_x
         placeholder = AbsolutePlaceholder(child)
         line_placeholders.append(placeholder)
-        waiting_children.append((index, placeholder))
+        waiting_children.append((index, placeholder, child))
         if child.style['position'] == 'absolute':
             absolute_boxes.append(placeholder)
         else:
@@ -543,7 +545,7 @@ def _out_of_flow_layout(context, box, containing_block, index, child,
         # To retrieve the real available space for floats, we must remove
         # the trailing whitespaces from the line
         non_floating_children = [
-            child_ for _, child_ in (children + waiting_children)
+            child_ for _, child_, _ in (children + waiting_children)
             if not child_.is_floated()]
         if non_floating_children:
             float_width -= trailing_whitespace_size(
@@ -558,10 +560,10 @@ def _out_of_flow_layout(context, box, containing_block, index, child,
                 context, child, containing_block, absolute_boxes, fixed_boxes,
                 bottom_space, skip_stack=None)
             if float_resume_at:
-                context.broken_out_of_flow.append(
-                    (child, containing_block, float_resume_at))
+                context.broken_out_of_flow[child] = (
+                    child, containing_block, float_resume_at)
+            waiting_children.append((index, new_child, child))
             child = new_child
-            waiting_children.append((index, child))
 
             # Translate previous line children
             dx = max(child.margin_width(), 0)
@@ -602,10 +604,15 @@ def _break_waiting_children(context, box, max_x, bottom_space,
         # fixed_boxes and other lists.
         waiting_children_copy = waiting_children.copy()
         while waiting_children_copy:
-            child_index, child = waiting_children_copy.pop()
+            child_index, child, original_child = waiting_children_copy.pop()
             if not child.is_in_normal_flow() or not can_break_inside(child):
                 continue
 
+            if initial_skip_stack and child_index in initial_skip_stack:
+                child_skip_stack = initial_skip_stack[child_index]
+            else:
+                child_skip_stack = None
+
             # Break the waiting child at its last possible breaking point.
             # TODO: The dirty solution chosen here is to decrease the
             # actual size by 1 and render the waiting child again with this
@@ -613,9 +620,10 @@ def _break_waiting_children(context, box, max_x, bottom_space,
             max_x = child.position_x + child.margin_width() - 1
             while max_x > child.position_x:
                 new_child, child_resume_at, _, _, _, _ = split_inline_level(
-                    context, child, child.position_x, max_x, bottom_space,
-                    None, box, absolute_boxes, fixed_boxes, line_placeholders,
-                    waiting_floats, line_children)
+                    context, original_child, child.position_x, max_x,
+                    bottom_space, child_skip_stack, box, absolute_boxes,
+                    fixed_boxes, line_placeholders, waiting_floats,
+                    line_children)
                 if child_resume_at:
                     break
                 max_x -= 1
@@ -628,43 +636,9 @@ def _break_waiting_children(context, box, max_x, bottom_space,
                 # May be None where we have an empty TextBox.
                 assert isinstance(child, boxes.TextBox)
             else:
-                children.append((child_index, new_child))
+                children.append((child_index, new_child, child))
 
-            # As this child has already been broken following the original
-            # skip stack, we have to add the original skip stack to the
-            # partial skip stack we get after the new rendering.
-
-            # Combining skip stacks is a bit complicated. We have to:
-            # - set `child_index` as the first number
-            # - append the new stack if it's an absolute one
-            # - otherwise append the combined stacks
-            #   (resume_at + initial_skip_stack)
-
-            # extract the initial index
-            if initial_skip_stack is None:
-                current_skip_stack = None
-                initial_index = 0
-            else:
-                (initial_index, current_skip_stack), = (
-                    initial_skip_stack.items())
-            # child_resume_at is an absolute skip stack
-            if child_index > initial_index:
-                return {child_index: child_resume_at}
-
-            # combine the stacks
-            current_resume_at = child_resume_at
-            stack = []
-            while current_skip_stack and current_resume_at:
-                (skip, current_skip_stack), = current_skip_stack.items()
-                (resume, current_resume_at), = current_resume_at.items()
-                stack.append(skip + resume)
-                if resume != 0:
-                    break
-            resume_at = current_resume_at
-            while stack:
-                resume_at = {stack.pop(): resume_at}
-            # insert the child index
-            return {child_index: resume_at}
+            return {child_index: child_resume_at}
 
     if children:
         # Too wide, can't break waiting children and the inline is
@@ -803,7 +777,7 @@ def split_inline_box(context, box, position_x, max_x, bottom_space, skip_stack,
                     break
 
             position_x = new_position_x
-            waiting_children.append((index, new_child))
+            waiting_children.append((index, new_child, child))
 
         waiting_floats.extend(child_waiting_floats)
         if resume_at is not None:
@@ -817,7 +791,7 @@ def split_inline_box(context, box, position_x, max_x, bottom_space, skip_stack,
     # Reorder inline blocks when direction is rtl
     if box.style['direction'] == 'rtl' and len(children) > 1:
         in_flow_children = [
-            box_child for _, box_child in children
+            box_child for _, box_child, _ in children
             if box_child.is_in_normal_flow()]
         position_x = in_flow_children[0].position_x
         for child in in_flow_children[::-1]:
@@ -827,7 +801,7 @@ def split_inline_box(context, box, position_x, max_x, bottom_space, skip_stack,
 
     is_end = resume_at is None
     new_box = box.copy_with_children(
-        [box_child for index, box_child in children])
+        [box_child for index, box_child, _ in children])
     new_box.remove_decoration(start=not is_start, end=not is_end)
     if isinstance(box, boxes.LineBox):
         # We must reset line box width according to its new children
@@ -924,7 +898,7 @@ def split_text_box(context, box, available_width, skip, is_line_start=True):
         box.pango_layout = layout
         # "The height of the content area should be based on the font,
         #  but this specification does not specify how."
-        # http://www.w3.org/TR/CSS21/visudet.html#inline-non-replaced
+        # https://www.w3.org/TR/CSS21/visudet.html#inline-non-replaced
         # We trust Pango and use the height of the LayoutLine.
         box.height = height
         # "only the 'line-height' is used when calculating the height
@@ -947,7 +921,7 @@ def split_text_box(context, box, available_width, skip, is_line_start=True):
         preserved_line_break = (
             (length != resume_index) and between.strip(' '))
         if preserved_line_break:
-            # See http://unicode.org/reports/tr14/
+            # See https://unicode.org/reports/tr14/
             # \r is already handled by process_whitespace
             line_breaks = ('\n', '\t', '\f', '\u0085', '\u2028', '\u2029')
             assert between in line_breaks, (
@@ -1164,7 +1138,8 @@ def add_word_spacing(context, box, justification_spacing, x_advance):
         nb_spaces = count_spaces(box)
         if nb_spaces > 0:
             layout = create_layout(
-                box.text, box.style, context, -inf, box.justification_spacing)
+                box.text, box.style, context, max_width=None,
+                justification_spacing=box.justification_spacing)
             layout.deactivate()
             extra_space = justification_spacing * nb_spaces
             x_advance += extra_space
@@ -1185,7 +1160,7 @@ def add_word_spacing(context, box, justification_spacing, x_advance):
 
 
 def is_phantom_linebox(linebox):
-    # See http://www.w3.org/TR/CSS21/visuren.html#phantom-line-box
+    # See https://www.w3.org/TR/CSS21/visuren.html#phantom-line-box
     for child in linebox.children:
         if isinstance(child, boxes.InlineBox):
             if not is_phantom_linebox(child):
